/*
   Drawer.h - Library for Drawer.
   Created by Raoul Minder, January, 2015.
   Released into the public domain.
 */

#include "Arduino.h"
#include "Motor.h"
#include "Reed.h"
#include "IrSettings.h"
#include "Logger.h"
#include "Drawer.h"

Drawer::Drawer(String name, int motorInput1Pin, int motorInput2Pin, int motorEnablePin, int reedFrontPin, int reedBackPin, String irCode1, String irCode2)
{
	_name = name;
	_state = closedState;
	_motor = new Motor(motorInput1Pin, motorInput2Pin, motorEnablePin);
	_reedFront = new Reed(name + "->ReedFront", reedFrontPin);
	_reedBack = new Reed(name + "->ReedBack", reedBackPin);
	_irSettings = new IrSettings(name, irCode1, irCode2);
	_logger = new Logger(name);
}

void Drawer::setup()
{
	pinMode(getReedFrontPin(), INPUT); 
	pinMode(getReedBackPin(), INPUT); 
	digitalWrite(getReedFrontPin(), HIGH); 
	digitalWrite(getReedBackPin(), HIGH); 
}

Drawer::drawerState Drawer::getState()
{
	_logger->log("getState() = " + String(_state));
	return _state;
}

bool Drawer::readIr(String code)
{
	bool result = _irSettings->read(code);
	if(result) _logger->log("readIr(" + code + ") = " + String(result));
	return result;
}

bool Drawer::readFront()
{
	bool result = _reedFront->read();
	//_logger->log("readFront() = " + String(result));
	return result;
}

int Drawer::getReedFrontPin()
{
	int result = _reedFront->getInputPin();
	_logger->log("getInputPin() = " + String(result));
	return result;
}

bool Drawer::readBack()
{
	bool result = _reedBack->read();
	//_logger->log("readBack() = " + String(result));
	return result;
}

int Drawer::getReedBackPin()
{
	int result = _reedBack->getInputPin();
	_logger->log("getReedBackPin() = " + String(result));
	return result;
}

void Drawer::startOpening()
{
	if(_state == startOpeningState)
		return;
	
	_state = startOpeningState;
	_logger->log("startOpening");
	_motor->turnLeft(255);
	
	_logger->log(_motor->getSetup());
}

void Drawer::continueOpening()
{
	if(_state == continueOpeningState)
		return;
	
	_state = continueOpeningState;
	_logger->log("continueOpening");
}

void Drawer::stopOpening()
{
	if(_state == stopOpeningState)
		return;
	
	_state = stopOpeningState;
	_logger->log("stopOpening");
	_motor->stop();
	_state = openedState;
	_logger->log("opened");
}

void Drawer::startClosing()
{
	if(_state == startClosingState)
		return;
	
	_state = startClosingState;
	_logger->log("startClosing");
	_motor->turnRight(255);
}

void Drawer::continueClosing()
{
	if(_state == continueClosingState)
		return;
	
	_state = continueClosingState;
	_logger->log("continueClosing");
}

void Drawer::stopClosing()
{
	if(_state == continueOpeningState)
		return;
	
	_state = stopClosingState;
	_logger->log("stopClosing");
	_motor->stop();
	_state = closedState;
	_logger->log("closed");
}

Motor* Drawer::getMotor()
{
	return _motor;
}

void Drawer::getMotorSetup()
{
	_logger->log(_motor->getSetup());
}
