/*
   Drawer.h - Library for Drawer.
   Created by Raoul Minder, January, 2015.
   Released into the public domain.
 */
 
#ifndef Drawer_h
#define Drawer_h

#include "Arduino.h"
#include "Motor.h"
#include "Reed.h"
#include "IrSettings.h"
#include "Logger.h"

class Drawer
{
	public:
		Drawer(String name, int motorInput1Pin, int motorInput2Pin, int motorEnablePin, int reedFrontPin, int reedBackPin, String irCode1, String irCode2);

		typedef enum {
			closedState,
			startOpeningState,
			continueOpeningState,
			stopOpeningState,
			openedState,
			startClosingState,
			continueClosingState,
			stopClosingState
		} drawerState;
		Drawer::drawerState getState();

		void startOpening();
		void continueOpening();
		void stopOpening();
		void startClosing();
		void continueClosing();
		void stopClosing();

		bool readFront();
		int getReedFrontPin();

		bool readBack();
		int getReedBackPin();

		bool readIr(String);

		void setup();
		
		Motor* getMotor();
		
		void getMotorSetup();

	private:
		String _name;

		Drawer::drawerState _state;

		IrSettings *_irSettings;
		Motor *_motor;
		Reed *_reedFront;
		Reed *_reedBack;
		Logger *_logger;
};

#endif
