/*
   Schubladenzug.ino
   Created by Raoul Minder, January, 2015.
   Released into the public domain.
 */
 
// Includes IR-Library
#include <IRremote.h>
#include <IrSettings.h>
#include <Motor.h>
#include <Reed.h>
#include <Logger.h>
#include <Drawer.h>

// Drawer
// int motorInput1Pin, int motorInput2Pin, int motorEnablePin, int reedFrontPin, int reedBackPin, String irCode1, String irCode2
Drawer drawerTop("DrawerTop", 11, 10, 9, 8, 7, "78b", "f8b");
Drawer drawerBottom("DrawerBottom", 6, 5, 3, 4, 2, "fae", "7ae");

// IR
const int IrPin = 12;
IRrecv irrecv(IrPin);
decode_results results;

// LED 
const int LedPin = 13;

void setup()
{
  // Console
  Serial.begin(9600);
  
  // IR
  irrecv.enableIRIn();
  
  // LED
  pinMode (LedPin, OUTPUT); 
  
  // Drawer
  drawerTop.setup();
  drawerBottom.setup();
}
 
void loop()
{
  //IR
  if (irrecv.decode(&results)) {
    String irCode = String(results.value, HEX);
    
    if(drawerTop.readIr(irCode))
    {
      switch(drawerTop.getState())
      {
        case Drawer::closedState:
          drawerTop.startOpening();
          break;
        case Drawer::openedState:
          drawerTop.startClosing();
          break;
      }
    }
    
    if(drawerBottom.readIr(irCode))
    {
      switch(drawerBottom.getState())
      {
        case Drawer::closedState:
          drawerBottom.startOpening();
          break;
        case Drawer::openedState:
          drawerBottom.startClosing();
          break;
      }
    }
    
    irrecv.resume(); // Receive the next value
  }

  // REED_BACK
  if(drawerTop.readBack())
  {
      Drawer::drawerState state = drawerTop.getState();
      switch(state)
      {
        case Drawer::startOpeningState:
          drawerTop.continueOpening();
          break;
        case Drawer::continueClosingState:
          drawerTop.stopClosing();
          break;
        default:
          throwException("drawerTop.readBack()" + String(state));
      }
  }

  if(drawerBottom.readBack())
  {
      Drawer::drawerState state = drawerBottom.getState();
      switch(state)
      {
        case Drawer::startOpeningState:
          drawerBottom.continueOpening();
          break;
        case Drawer::continueClosingState:
          drawerBottom.stopClosing();
          break;
        default:
          throwException("drawerBottom.readBack()" + String(state));
      }
  }

  // REED_FRONT
  if(drawerTop.readFront())
  {
      Drawer::drawerState state = drawerTop.getState();
      switch(state)
      {
        case Drawer::continueOpeningState:
          drawerTop.stopOpening();
          break;
        case Drawer::startClosingState:
          drawerTop.continueClosing();
          break;
        default:
          throwException("drawerTop.readFront()" + String(state));
      }
  }

  if(drawerBottom.readFront())
  {
      Drawer::drawerState state = drawerBottom.getState();
      switch(state)
      {
        case Drawer::continueOpeningState:
          drawerBottom.stopOpening();
          break;
        case Drawer::startClosingState:
          drawerBottom.continueClosing();
          break;
        default:
          throwException("drawerBottom.readFront()" + String(state));
      }
  }

  delay(100);
}

void throwException(String msg)
{
  Serial.println("Exception: " + msg);
  
  drawerTop.getMotor()->stop();
  drawerBottom.getMotor()->stop();
  
  while(true)
  {
    digitalWrite(13, HIGH);
    delay(100);
    digitalWrite(13, LOW);
    delay(100);
  }
}

